// (C) Copyright 2011-2013 Hewlett-Packard Development Company, L.P.
/**
 * @type {MasterGridView}
 */
define(['hp/core/LinkTargetBuilder', 'hp/core/Localizer', 'hp/core/Style', 'jquery'],
function (linkTargetBuilder, localizer, style) {
"use strict";

    var MasterGridView = (function () {

        // consts can go here
        var MASTER_PANE = '.hp-master-pane';
        var DETAILS_PANE = '.hp-details-pane';
        var GRID = '.hp-master-grid';
        var ITEM = '.hp-master-grid-item';
        var SELECTED = 'hp-selected';
        var KEY_DOWN = 40;
        var KEY_UP = 38;

        /**
         * @constructor
         */
        function MasterGridView() {

            //
            // private class instance variables
            //
            var presenter = null;
            var resource = null;
            var page = null;
            var pane = null;
            var renderer = null;
            var multiSelect = false;
            var manualScrollTo = -1;
            var layoutTimer = null;
            var relayoutTimer = null;
            var addHelp = null;
            var sorter = null;
            
            function fireRelayout() {
                // we delay so any CSS animation can finish
                clearTimeout(relayoutTimer);
                relayoutTimer = setTimeout(function () {
                    page.trigger('relayout');
                }, style.animationDelay());
            }

            function clearSelection() {
                $(ITEM, pane).removeClass(SELECTED);
            }

            function setSelection(selection) {
                if (selection.uris && selection.uris.length > 0) {
                    $.each($(ITEM), function() {
                        var item = this;
                        var indexResult = $.data(item, 'hpIndexResult');

                        if ((indexResult) &&
                            ($.inArray(indexResult.uri, selection.uris) != -1)) {
                            $(item).addClass(SELECTED);
                        }
                    });
                }
            }
            
            function scrollToSelection() {
                var scrollTo = manualScrollTo;
                var reset = false;
                
                if (scrollTo === -1) {
                    var item = $(ITEM + '.' + SELECTED).first();
                    if (item.length > 0) {
                        if ($(item).position().top < $(GRID).position().top) {
                            scrollTo =  $(GRID).scrollTop() +
                                item.position().top - $(GRID).position().top;                                                
                        } else if (($(item).position().top + $(item).outerHeight()) >
                            ($(GRID).position().top + $(GRID).innerHeight())) {
                            // make sure we don't scroll up so we can't see the top of the item
                            scrollTo =  Math.min(
                                $(GRID).scrollTop() +
                                    item.position().top - $(GRID).position().top,
                                $(GRID).scrollTop() + item.position().top -
                                    $(GRID).innerHeight() + item.outerHeight()); 
                        }
                    }
                    reset = true;
                }
                
                if (scrollTo !== -1) {
                    $(GRID).animate({'scrollTop' : scrollTo}, 20);
                }
                
                setTimeout(function () {
                    if (reset) {
                        manualScrollTo = -1; // reset since we just scrolled
                    }
                }, 100);
            }

            function resetSelection(selection) {
                clearSelection();
                setSelection(selection);
                scrollToSelection();
            }
            
            function onItemClick(event) {
                // get index result from object and call presenter
                var item = $(event.target).closest('.hp-master-grid-item');
                var indexResult = $(item).data('hpIndexResult');
                if (multiSelect && (event.ctrlKey || event.metaKey)) {
                    presenter.toggleSelectIndexResult(indexResult);
                } else if (multiSelect && event.shiftKey) {
                    presenter.selectContiguousIndexResults(indexResult);
                } else {
                    presenter.selectIndexResult(indexResult);
                }
                $(GRID).focus();
            }
            
            function setEmptyMessage() {
                var item = $('<li></li>').addClass('hp-master-grid-control');
                var empty = $('<div></div>').addClass('hp-master-grid-empty').
                    text(presenter.getEmptyMessage());
                item.append(empty);
                if (presenter.haveContacted() && ! presenter.haveSome() && addHelp) {
                    item.append('<div class="hp-help">' + addHelp + '</div>');
                }
                $(GRID, pane).append(item);
            }
            
            function calcTargetHeight() {
                var result;
                // use all height available in the pane that isn't used by other
                // elements in the pane, 20 provides padding
                result = pane.height() - 20;
                $.each(pane.children(), function (index, child) {
                    if ($(child).is(':visible') &&
                        ! $(child).hasClass('hp-master-grid') &&
                        ! $(child).hasClass('ui-resizable-handle')) {
                       result -= $(child).outerHeight();
                    }
                });
                return result;
            }
            
            function layout() {
                var currentWidth = pane.width();
                var paddingWidth = (pane.outerWidth() - currentWidth);
                var targetHeight = calcTargetHeight();
                // what column sizes should we align to?
                var quanta = Math.floor($(window).width() / 12);
                var targetWidth, requestedItemWidth, itemWidth;
                
                // How much width do the items want?
                // Determine by shrinking the grid so the items can report what they need
                $(GRID, pane).css({'width': quanta + style.scrollBarWidth()});
                $(ITEM, pane).css({'width': ''});
                //widths = $(ITEM, pane).map(function (index, item) {
                //    return $(item).outerWidth();
                //}).get();
                // pick the largest
                //requestedItemWidth = Math.max.apply(null, widths);
                requestedItemWidth = $(ITEM, pane).first().outerWidth();
                var itemMarginWidth =
                    $(ITEM, pane).first().outerWidth(true) - requestedItemWidth;
                
                // modulate to discrete window column sizes
                itemWidth = quanta * Math.floor((requestedItemWidth + (quanta / 2)) / quanta);
                
                // figure out what to size the master pane width to
                if (pane.hasClass('hp-resized')) {
                    // user has set an explit size
                    targetWidth = currentWidth;
                    var availableTargetWidth = targetWidth - style.scrollBarWidth();
                    // fill available space
                    var delta = (availableTargetWidth / (itemWidth + itemMarginWidth));
                    itemWidth = Math.floor(availableTargetWidth / Math.floor(delta)) - itemMarginWidth;
                    // make sure the items aren't clipped
                    itemWidth = Math.min(itemWidth, availableTargetWidth);
                } else {
                    // one column and leave room for scrollbars
                    targetWidth = itemWidth + itemMarginWidth + style.scrollBarWidth();
                    // also check width of header items
                    $.each($('.hp-master-header h1', pane).children(), function (index, child) {
                        if ($(child).is(':visible')) {
                           targetWidth = Math.max($(child).outerWidth(true), targetWidth);
                        }
                    });
                }
                
                //console.log('!!! MGV', (new Date()).getTime(), currentWidth, '->', targetWidth, itemWidth);
                
                $(GRID, pane).css({'width': ''});
                $(ITEM, pane).css({'width': itemWidth});
                $(GRID).css('height', targetHeight);
                if (currentWidth != targetWidth &&
                    ! pane.hasClass('hp-resized')) {
                    // adjust width since master pane is box sized for content-box
                    // align with container padding
                    pane.css({'height': '', 'width': targetWidth});
                    $(DETAILS_PANE, page).css('left', targetWidth + paddingWidth);
                    fireRelayout();                        
                }
            }

            function resetSort() {
                // reset sort indicator:
                // set the sorter to the presenter sort value if available, otherwise set the presenter
                // sort value to the sorter value
                if (sorter) {
                    var sort = presenter.getSort();
                    if (sort) {
                        $('.hp-sort', sorter).hpSelect('set', {ids: [sort.name], direction: sort.direction});
                    } else {
                        var selection = $('.hp-sort', sorter).hpSelect('get');
                        if (selection && selection.ids) {
                            presenter.setSort(selection.ids, selection.direction ? selection.direction : 'asc');
                        }
                    }
                }
            }

            /**
             * @private
             * @param {Object} data the IndexResults
             */
            function onIndexResultsChange(indexResults) {
                var item;
                if (indexResults && renderer) {
                    var expandedUris = $(ITEM + '.hp-expanded', pane).map(function () {
                        return $(this).data('hpIndexResult').uri;
                    }).get();
                
                    $(GRID, pane).empty();
                    $(GRID, pane).removeClass('hp-empty');
                    
                    $.each(indexResults.members, function (index, result) {
                        item = $(renderer(result));
                        $(GRID, pane).append(item);
                        item.data('hpIndexResult', result);
                        if (-1 !== $.inArray(result.uri, expandedUris)) {
                            item.addClass('hp-expanded');
                        }
                        item.click(onItemClick);
                        $('.hp-master-item-expander', item).click(function (ev) {
                            $(this).closest('.hp-master-grid-item').
                                toggleClass('hp-expanded');
                            ev.stopPropagation();
                        });
                    });
                    
                    if (indexResults.total > indexResults.count) {
                        // we could show more. Add control to add more
                        $(GRID, pane).append(
                            '<li class="hp-master-grid-control">' +
                            '<a class="hp-master-show-more">' +
                            localizer.getString('core.master.showMore') +
                            '</a></li>');
                        $('.hp-master-show-more', pane).click(function () {
                            $(this).replaceWith('');
                            presenter.showMore();
                        });
                    } else if (indexResults.total === 0) {
                        setEmptyMessage();
                    }

                    resetSort();
                    layout();
                    resetSelection(presenter.getSelection());
                }
            }

            function onIndexResultsError(errorInfo) {
                if (pane) {
                    $(GRID, pane).addClass('hp-empty');
                }
            }
            
            // align selection classes with uris
            function onSelectionChange(selection) {
                resetSelection(selection);
            }
            
            function onKeyDown(event) {
                if ($(GRID).is(":focus")) {
                    var keyCode = (event.which ? event.which : event.keyCode);
                    if (keyCode === KEY_DOWN) {
                        manualScrollTo = -1;
                        $('.hp-master-grid-item.hp-selected').next().trigger('click');
                        event.stopPropagation();
                        return false;
                    } else if (keyCode === KEY_UP) {
                        manualScrollTo = -1;
                        $('.hp-master-grid-item.hp-selected').prev().trigger('click');
                        event.stopPropagation();
                        return false;
                    }
                }
            }
            
            function onResize(event) {
                if (event.target == window) {
                    clearTimeout(layoutTimer);
                    layoutTimer = setTimeout(layout, 50);
                } else if (event.target == page[0]) {
                    clearTimeout(layoutTimer);
                    layout();
                }
            }

            function onSortChange(event, sort) {
                presenter.setSort(sort.id, sort.direction);
            }

            /**
             * @public
             * Stop the timer polling of the index service.
             */
            this.pause = function () {
                presenter.off("indexResultsChange", onIndexResultsChange);
                presenter.off("indexResultsError", onIndexResultsError);
                presenter.off("selectionChange", onSelectionChange);
                $(window).off('resize', onResize);
                page.off('relayout', layout);
                if (sorter) {
                    sorter.hide();
                }
            };

            /**
             * @public
             * Resume the timer polling of the index service.
             */
            this.resume = function () {
                manualScrollTo = -1;
                presenter.on("indexResultsChange", onIndexResultsChange);
                presenter.on("indexResultsError", onIndexResultsError);
                presenter.on("selectionChange", onSelectionChange);
                $(window).on('resize', onResize);
                page.on('relayout', layout);
                if (sorter) {
                    resetSort();
                    sorter.show();
                }
            };
            
            /**
             * @public
             * Intialize the view.
             */
            this.init = function (presenterArg, args) {
                presenter = presenterArg;
                resource = args.resource;
                page = args.page;
                pane = $(MASTER_PANE, page);
                renderer = args.gridItemRenderer;
                multiSelect = args.multiSelect;
                addHelp = args.addHelp;
                sorter = $('.hp-sub-nav .hp-master-grid-sort', page);

                if (sorter && sorter.length > 0) {
                    sorter.hide();
                    $('.hp-master-add', pane).after(sorter);
                    $('.hp-sort', sorter).hpSelect({sorting: true}).change(onSortChange);
                }
                
                setEmptyMessage();
                
                $(GRID).scroll(function () {
                    // stop auto scrolling if the user scrolls
                    manualScrollTo = $(GRID).scrollTop();
                });
                $(GRID).on('focus', function (event) {
                    if (event.target == $(GRID)[0]) {
                        $(document).off('keydown', onKeyDown);  
                        $(document).on('keydown', onKeyDown);
                    }
                });
                $(GRID).on('blur', function (event) {
                    if (event.target == $(GRID)[0]) {
                        $(document).off('keydown', onKeyDown);  
                    }
                });

            };
        }

        return MasterGridView;

    }());

    return MasterGridView;
});
